from typing import Union, Hashable, Dict

import xarray as xr
import astropy.units as u

from .quantity_array import QuantityArray


class QuantitySet(xr.Dataset):

    __slots__ = (
        "_attrs",
        "_cache",
        "_coord_names",
        "_dims",
        "_encoding",
        "_close",
        "_indexes",
        "_variables",
    )

    _unit = {}

    def __init__(self, data_vars, *args, **kwargs):
        super().__init__(data_vars, *args, **kwargs)
        for k, v in data_vars.items():
            try:
                if not isinstance(v.unit, u.UnitBase):
                    raise TypeError
                unit = {k: v.unit}
                self._unit.update(unit)
            except (AttributeError, TypeError):
                pass

    @property
    def unit(self) -> Dict[str, u.Unit]:
        return self._unit

    @unit.setter
    def unit(self, unit: Dict[str, Union[str, u.Unit]]) -> None:
        self._unit.update(unit)

    # override
    def _construct_dataarray(self, name: Hashable) -> QuantityArray:
        da = super()._construct_dataarray(name)
        try:
            return QuantityArray(da, unit=self._unit[name])
        except KeyError:
            return QuantityArray(da)



# flake8: noqa

import pkg_resources

try:
    __version__ = pkg_resources.get_distribution("xarray-quantity").version
except pkg_resources.DistributionNotFound:
    __version__ = "0.1.1"

from .quantity_array import QuantityArray
from .quantity_set import QuantitySet



from typing import Any, Union

import xarray as xr
import astropy.units as u


class QuantityArray(xr.DataArray):

    __slots__ = (
        "_cache",
        "_coords",
        "_close",
        "_indexes",
        "_name",
        "_variable",
        "_unit",
    )

    def __init__(self, *args, **kwargs):
        try:
            self._unit = u.Unit(kwargs.pop("unit"))
        except KeyError:
            self._unit = None
        super().__init__(*args, **kwargs)

    @property
    def unit(self) -> u.Unit:
        return self._unit

    @unit.setter
    def unit(self, new_unit: Union[str, u.Unit]) -> None:
        self._unit = u.Unit(new_unit)

    # override
    @property
    def data(self) -> Any:
        if self._unit:
            return self.variable.data << self._unit
        return self.variable.data

    # override
    @data.setter
    def data(self, value: Any) -> None:
        self.variable.data = value
        if isinstance(value, u.Quantity):
            self._unit = value.unit



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xarray_quantity']

package_data = \
{'': ['*']}

install_requires = \
['astropy>=4.2.1,<5.0.0', 'xarray>=0.18.2,<0.19.0']

setup_kwargs = {
    'name': 'xarray-quantity',
    'version': '0.1.1',
    'description': 'xarray extension which supports astropy quantities.',
    'long_description': '# xarray-quantity\n\n[![PyPI](https://img.shields.io/pypi/v/PACKAGENAME.svg?label=PyPI&style=flat-square)](https://pypi.org/pypi/xarray-quantity/)\n[![Python](https://img.shields.io/pypi/pyversions/PACKAGENAME.svg?label=Python&color=yellow&style=flat-square)](https://pypi.org/pypi/xarray-quantity/)\n[![Test](https://img.shields.io/github/workflow/status/USERNAME/PACKAGENAME/Test?logo=github&label=Test&style=flat-square)](https://github.com/KaoruNishikawa/xarray-quantity/actions)\n[![License](https://img.shields.io/badge/license-MIT-blue.svg?label=License&style=flat-square)](LICENSE)\n\nxarray extension which supports astropy quantities.\n\n## Features\n\nThis library provides:\n\n- xarray DataArray and Dataset with units.\n\n## Installation\n\n```shell\npip install xarray-quantity\n```\n\n## Usage\n\n### QuantityArray\n\nTo create a DataArray with units, use `QuantityArray` class. The arguments are compatible with DataArray, except the keyword argument `unit`.\n\n```python\n>>> qa = QuantityArray([1, 2, 3, 4, 5], unit="km")\n>>> qa.data\n[1, 2, 3, 4, 5] km\n>>> qa.unit\nkm\n```\n\n### QuantitySet\n\nTo create a Dataset with units, use `QuantitySet` class. This class also has compatibility with xarray\'s Dataset.\n\n```python\n>>> arrays = {\n...     "qa1": QuantityArray([1, 2, 3, 4, 5], unit="km/s"),\n...     "qa2": QuantityArray([11, 12, 13, 14, 15]),\n...     "da3": xr.DataArray([111, 112, 113, 114, 115])\n... }\n>>> qs = QuantitySet(arrays)\n>>> qs.qa1.data\n[1, 2, 3, 4, 5] km / s\n>>> qs.qa2\nxarray.QuantityArray \'qa2\' (dim_0: 5)\n<Quantity [11., 12., 13., 14., 15.] km / s>\nCoordinates: (0)\nAttributes: (0)\n```\n\n---\n\nThis library is using [Semantic Versioning](https://semver.org).\n',
    'author': 'KaoruNishikawa',
    'author_email': 'k.nishikawa@a.phys.nagoya-u.ac.jp',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/KaoruNishikawa/xarray-quantity',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
